#include "stdafx.h"
#include "ButtonIcon.h"

ButtonIcon::ButtonIcon(const string &name)
{
	defaulticonSize = GetxRCInt(name, "IconSize", 16, 8, 64);
	defaulticonX = GetxRCString(name, "IconX", "0");
	defaulticonY = GetxRCString(name, "IconY", ("-" + IntToString(defaulticonSize/2) + "c") );
	
	defaulticon_saturation = GetxRCInt(name, "IconSaturationIntensity", 255, 0, 255);
	defaulticon_hueintensity = GetxRCInt(name, "IconHueIntensity", 0, 0, 255);
	defaulticon_huecolor = GetxRCColor(name, "IconHueColor", RGB(255, 255, 255));
}

ButtonIcon::~ButtonIcon()
{
}

void ButtonIcon::configure(const string &name)
{
	normaliconSize = GetxRCInt(name, "NormalIconSize", defaulticonSize, 8, 64);
	normaliconX = GetxRCString(name, "NormalIconX", defaulticonX);
	normaliconY = GetxRCString(name, "NormalIconY", defaulticonY);

	normalicon_saturation = GetxRCInt(name, "NormalIconSaturationIntensity", defaulticon_saturation, 0, 255);
	normalicon_hueintensity = GetxRCInt(name, "NormalIconHueIntensity", defaulticon_hueintensity, 0, 255);
	normalicon_huecolor = GetxRCColor(name, "NormalIconHueColor", defaulticon_huecolor);

	activeiconSize = GetxRCInt(name, "ActiveIconSize", normaliconSize, 8, 64);
	activeiconX = GetxRCString(name, "ActiveIconX", normaliconX);
	activeiconY = GetxRCString(name, "ActiveIconY", normaliconY);
	
	activeicon_saturation = GetxRCInt(name, "ActiveIconSaturationIntensity", normalicon_saturation, 0, 255);
	activeicon_hueintensity = GetxRCInt(name, "ActiveIconHueIntensity", normalicon_hueintensity, 0, 255);
	activeicon_huecolor = GetxRCColor(name, "ActiveIconHueColor", normalicon_huecolor);

	minimizediconSize = GetxRCInt(name, "MinimizedIconSize", normaliconSize, 8, 64);
	minimizediconX = GetxRCString(name, "MinimizedIconX", normaliconX);
	minimizediconY = GetxRCString(name, "MinimizedIconY", normaliconY);

	minimizedicon_saturation = GetxRCInt(name, "MinimizedIconSaturationIntensity", normalicon_saturation, 0, 255);
	minimizedicon_hueintensity = GetxRCInt(name, "MinimizedIconHueIntensity", normalicon_hueintensity, 0, 255);
	minimizedicon_huecolor = GetxRCColor(name, "MinimizedIconHueColor", normalicon_huecolor);

	flashingiconSize = GetxRCInt(name, "FlashingIconSize", normaliconSize, 8, 64);
	flashingiconX = GetxRCString(name, "FlashingIconX", normaliconX);
	flashingiconY = GetxRCString(name, "FlashingIconY", normaliconY);
	
	flashingicon_saturation = GetxRCInt(name, "FlashingIconSaturationIntensity", normalicon_saturation, 0, 255);
	flashingicon_hueintensity = GetxRCInt(name, "FlashingIconHueIntensity", normalicon_hueintensity, 0, 255);
	flashingicon_huecolor = GetxRCColor(name, "FlashingIconHueColor", normalicon_huecolor);
}

void ButtonIcon::apply(HDC hDC, int x, int y, int width, int height, int buttonType, HICON hIcon)
{
	if(hIcon == NULL)
		return;

	int iconX = 0;
	int iconY = 0;
	int iconSize = 16;

	int _saturation = 255;
	int _hueIntensity = 0;
	COLORREF _huecolor = RGB(255, 255, 255);

	if (buttonType == 0)
	{
		iconX = xParseCoord(normaliconX, 0, width);
		iconY = xParseCoord(normaliconY, 0, height);

		iconSize = normaliconSize;
		_saturation = normalicon_saturation;
		_hueIntensity = normalicon_hueintensity;
		_huecolor = normalicon_huecolor;
	}
	else if (buttonType == 1)
	{
		iconX = xParseCoord(activeiconX, 0, width);
		iconY = xParseCoord(activeiconY, 0, height);

		iconSize = activeiconSize;
		_saturation = activeicon_saturation;
		_hueIntensity = activeicon_hueintensity;
		_huecolor = activeicon_huecolor;
	}
	else if (buttonType == 2)
	{
		iconX = xParseCoord(minimizediconX, 0, width);
		iconY = xParseCoord(minimizediconY, 0, height);

		iconSize = minimizediconSize;
		_saturation = minimizedicon_saturation;
		_hueIntensity = minimizedicon_hueintensity;
		_huecolor = minimizedicon_huecolor;
	}
	else if (buttonType == 3)
	{
		iconX = xParseCoord(flashingiconX, 0, width);
		iconY = xParseCoord(flashingiconY, 0, height);

		iconSize = flashingiconSize;
		_saturation = flashingicon_saturation;
		_hueIntensity = flashingicon_hueintensity;
		_huecolor = flashingicon_huecolor;
	}

	if ( _saturation != 255 || _hueIntensity != 0 )
	{
		HDC hdcBuffer = CreateCompatibleDC(hDC);
		HBITMAP hbmBuffer = CreateBitmap(iconSize, iconSize, 1, 32, NULL);
		SelectObject(hdcBuffer, hbmBuffer);

		// Icon
		DrawIconEx(hdcBuffer, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);

		HDC hdcMask = CreateCompatibleDC(hdcBuffer);
		HBITMAP hbmMask = CreateCompatibleBitmap(hdcBuffer, iconSize, iconSize);
		SelectObject(hdcMask, hbmMask);

		// Mask of the icon
		DrawIconEx(hdcMask, 0, 0, hIcon, iconSize, iconSize, 0, NULL, DI_MASK);

		int iy = 0;
		int ix = 0;
		COLORREF cl = 0;
		BYTE r = 0;
		BYTE g = 0;
		BYTE b = 0;
		for(iy = 0; iy < iconSize; iy++)
		{
			for(ix = 0; ix < iconSize; ix++)
			{
				// Loop through all transparent pixels...
				// If the mask-pixel is white, then break
				if(!GetPixel(hdcMask, ix, iy))
				{
					cl = GetPixel(hdcBuffer, ix, iy);
					r = GetRValue(cl);
					g = GetGValue(cl);
					b = GetBValue(cl);

					// Saturation effect
					if(_saturation != 255)
					{
						BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

						r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
						g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
						b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
					}

					// Hue effect
					if(_hueIntensity != 0 )
					{
						// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
						// test normal
						r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_huecolor) * _hueIntensity + 255) >> 8);
						g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_huecolor) * _hueIntensity + 255) >> 8);
						b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_huecolor) * _hueIntensity + 255) >> 8);
					}

					SetPixelV(hdcBuffer, ix, iy, RGB(r, g, b));
				}
			}
		}

		BitBlt( hDC, x+iconX, y+iconY, iconSize, iconSize, hdcMask, 0, 0, SRCAND );
		// Combine the foreground with the background
		BitBlt( hDC, x+iconX, y+iconY, iconSize, iconSize, hdcBuffer, 0, 0, SRCPAINT );
			
		hbmMask = (HBITMAP) SelectObject(hdcMask, hbmMask);
		DeleteObject(hbmMask);
		DeleteDC(hdcMask);

		hbmBuffer = (HBITMAP) SelectObject(hdcBuffer, hbmBuffer);
		DeleteObject(hbmBuffer);
		DeleteDC(hdcBuffer);
	}
	else
		DrawIconEx(hDC, x+iconX, y+iconY, hIcon, iconSize, iconSize, 0, NULL, DI_NORMAL);
}

