#if !defined(AFX_TASKBAR_H__EBF1ABD2_1EA7_40AF_93CF_D4F2B4ECC8EE__INCLUDED_)
#define AFX_TASKBAR_H__EBF1ABD2_1EA7_40AF_93CF_D4F2B4ECC8EE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class DropTarget;

class TaskbarTexture;
class TaskbarButton;
class ButtonTexture;
class ButtonIcon;
class ButtonFont;

//Ballon Tooltip
//#ifndef TTS_BALLOON
//#define TTS_BALLOON 0x40
//#endif

//Alpha Transparency
#ifndef WS_EX_LAYERED
#define WS_EX_LAYERED           0x00080000L
#define LWA_COLORKEY            0x00000001L
#define ULW_COLORKEY            0x00000001L
#define LWA_ALPHA               0x00000002L
#define ULW_ALPHA               0x00000002L
#define ULW_OPAQUE              0x00000003L
#endif // WS_EX_LAYERED

#ifndef AC_SRC_ALPHA
#define AC_SRC_ALPHA            0x00000001L
#endif

#ifndef WM_MOUSEWHEEL
	#define WM_MOUSEWHEEL 0x020A
#endif

typedef vector<TaskbarButton *>::iterator TaskbarButtonIterator;

class Taskbar  
{

public:
	Taskbar(const string &name);
	virtual ~Taskbar();

	DropTarget*	pDropTarget;

	void load(HINSTANCE hInstance, HWND box = 0);
	void reconfigure(bool refresh = false);

	const HWND &gethWnd() const { return hWnd; }
	const string &getName() const { return name; }
	const HWND &getTooltip() const { return ToolTips; }
	const HWND &getBox() const { return box; }
	bool isAlwaysOnTop() const { return alwaysOnTop; }
	bool isVisible() const { return visible; }
	bool isStartHidden() const { return startHidden; }
	bool isLayeredFlagSet() const { return layeredFlag; }

	void move(int newx, int newy, int steps = 0, int time = 0, bool moveBy = false);
	void resize(int newwidth, int newheight, int steps = 0, int time = 0, bool resizeBy = false);
	void reposition(int newx, int newy, int newwidth, int newheight, int steps = 0, int time = 0, bool repositionBy = false);

	void setBox(HWND newparent);
	void setAlwaysOnTop(bool alwaysOnTop);
	void setAlpha(int newAlphaTransparency = 255);

	void SwitchToButton( LPCSTR direction );

	void repaint(bool invalidateCache = false);
	void updateRegion();

	void show();	
	void hide();

	bool useBigIcon;
	int lines_rows;
	int maxlines_rows;
	int displayButtonType;
	void CreateLayout();

	//For ReposButtons
	TaskbarButton *movecapture;

	//For Tooltips
	TaskbarButton *capture;

	//Access from TaskbarButton (FriendClass)
	bool noMinimizeOnClick;
	
	bool showIcon;
	bool showText;
	bool showToolTip;

	bool useGrouping;

	bool useHoverState;

	ButtonTexture *buttontexture;
	ButtonIcon *buttonicon;
	ButtonFont *buttonfont;

	HWND lastActive;

private:

	string name;

	bool alphaFade;
	int UpdateAlpha;
	int alphaFadeStep;
	int alphaFadeDelay;

	bool alphaInitialized;
	bool layeredFlag;
	int alphaTransparency;

	bool transparencyNeeded;

	HWND hWnd;
	HINSTANCE hInstance;
	HWND box;
	HWND ToolTips;

	bool mouseInside;
	int focusOnEnter;
	//int pressedModifierKey;

	vector<TaskbarButton *> taskbuttons;
	vector<TaskbarButton *> displayedtaskbuttons;

	HDC backgroundDC;
	HDC bufferDC;
	HBITMAP backgroundBitmap;
	HBITMAP bufferBitmap;
	bool inCriticalProcess;

	TaskbarTexture *taskbartexture;

	int actualx;
	int actualy;
	int actualwidth;
	int actualheight;

	int leftBorder;
	int topBorder;
	int rightBorder;
	int bottomBorder;

	int maxTaskWidth;
	int maxTaskHeight;
	int minTaskWidth;
	int minTaskHeight;

	bool autoNewLine;
	int autolines_rows;

	int direction;

	bool transparencyMode;

	bool loadInBox;
	bool startHidden;
	bool alwaysOnTop;
	bool visible;

	//Actions
	int lButtonAction;
	int rButtonAction;
	int mButtonAction;

	string taskbarLButtonDown;
	string taskbarLButtonUp;
	string taskbarRButtonDown;
	string taskbarRButtonUp;
	string taskbarMButtonDown;
	string taskbarMButtonUp;
	string taskbarMWheelDown;
	string taskbarMWheelUp;
	string taskbarEnter;
	string taskbarLeave;
	string taskbarAdd;
	string taskbarRemove;
	string taskbarMove;
	string taskbarResize;

	string taskbarLineAdd;
	string taskbarLineRemove;

	//General Button Settings
	int timerRefresh;
	int flashRefresh;
	int xSpacing;
	int ySpacing;

protected:

	virtual void onLButtonDown(int x, int y);
	virtual void onLButtonUp(int x, int y);
	virtual void onMButtonDown(int x, int y);
	virtual void onMButtonUp(int x, int y);
	virtual void onRButtonDown(int x, int y);
	virtual void onRButtonUp(int x, int y);
	virtual void onWheelDown(int x, int y);
	virtual void onWheelUp(int x, int y);
	virtual void onMouseEnter();
	virtual void onMouseLeave();
	virtual void onMouseMove(int x, int y);
	virtual void onPaint(HDC hDC);
	virtual void onWindowPosChanged(WINDOWPOS *windowPos);
	virtual void onSize(int width, int height);
	virtual void onTimer(int timerID);

private:

	void onRedraw( HWND hApplication, bool fFlash );
	void onWindowActivated( HWND hApplication, BOOL fFullScreen);
	void onWindowCreated( HWND hApplication);
	void onWindowDestroyed( HWND hApplication);

	void updateHint(HWND hWnd, LPSTR caption, bool updatesize, bool remove = false);

private:
	BOOL onWindowMessage( UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult );

private:   // functions

	BOOL UpdateWindowTransparent(HWND hWnd, int factor);

	TaskbarButton *SearchForButtonByHandle( HWND search_hWnd );
	TaskbarButton *SearchForButtonByClass( HWND search_hWnd );
	TaskbarButton *ButtonAtPoint( int x, int y );

private:   // statics
	static BOOL WINAPI EnumWindowsProc( HWND hWnd, LPARAM lParam );

public:
	static LRESULT WINAPI windowProcedure( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );

};

#endif // !defined(AFX_TASKBAR_H__EBF1ABD2_1EA7_40AF_93CF_D4F2B4ECC8EE__INCLUDED_)
